@@ -15,8 +15,8 @@ from fresco.util.wsgi import unicode_to_environ
 
 from fresco.exceptions import ResponseException
 from fresco.requestcontext import context
-from fresco.routing import GET, ExtensiblePattern, RouteCollection, \
-                           RouteKwarg, RouteNotFound
+from fresco.routing import (GET, ExtensiblePattern, RouteCollection,
+                            RouteNotFound)
 from fresco.options import Options
 
 __all__ = ('FrescoApp', 'urlfor')
@@ -49,8 +49,9 @@ class FrescoApp(RouteCollection):
         #: :meth:`add_middleware`.
         self.core_middleware = [partial(context_middleware, frescoapp=self)]
 
-        #: The WSGI application. Replaced every time new middleware is added.
-        self._wsgi_app = self._make_wsgi_app()
+        #: The WSGI application. This is generated when the first request is
+        #: made.
+        self._wsgi_app = None
 
         #: An options dictionary, for arbitrary application variables or
         #: configuration
@@ -60,12 +61,12 @@ class FrescoApp(RouteCollection):
 
         #: Sent when a request matches a route, immediately before the view is
         #: invoked.
-        #: Receivers will be passed: route, view, view_args, view_kwargs
+        #: Receivers will be passed: app, route, view, request
         self.route_matched = self.signal_ns.signal('route_matched')
 
-        #: Sent Sent after a view function is invoked, before the response
+        #: Sent after a view function is invoked, before the response
         #: object is output.
-        #: Receivers will be passed: view, request and response
+        #: Receivers will be passed: app, view, request and response
         self.view_finished = self.signal_ns.signal('view_finished')
 
         if views:
@@ -77,6 +78,8 @@ class FrescoApp(RouteCollection):
         """\
         Call the app as a WSGI application
         """
+        if self._wsgi_app is None:
+            self._wsgi_app = self._make_wsgi_app()
         return self._wsgi_app(environ, start_response)
 
     def __str__(self):
@@ -90,33 +93,27 @@ class FrescoApp(RouteCollection):
         return s
 
     def view(self, routecollection=None, request=None, path=None, method=None):
-        routecollection = routecollection or self
-        request = request or context.request
-        method = method or request.method
-        path = path or request.path_info
-        error_response = None
+        try:
+            routecollection = routecollection or self
+            request = request or context.request
+            method = method or request.method
+            path = path or request.path_info
+            error_response = None
 
-        context.app = self
-        environ = request.environ
-        environ['fresco.app'] = self
+            context.app = self
+            environ = request.environ
+            environ['fresco.app'] = self
 
-        if path:
-            path = normpath(path)
-        else:
-            path = '/'
+            if path:
+                path = normpath(path)
+            else:
+                path = '/'
+        except ResponseException as e:
+            return e.response
 
-        for traversal in routecollection.get_routes(path, method):
-
-            route = traversal.route
+        for traversal in routecollection.get_routes(path, method, request):
             try:
-                if route.predicate and not route.predicate(request):
-                    continue
-
-                for k, v in traversal.kwargs.items():
-                    if isinstance(v, RouteKwarg):
-                        v = v(request)
-                    traversal.kwargs[k] = v
-
+                route = traversal.route
                 environ['wsgiorg.routing_args'] = (traversal.args,
                                                    traversal.kwargs)
                 view = route.getview()
@@ -137,7 +134,8 @@ class FrescoApp(RouteCollection):
                     return e.response
                 error_response = error_response or e.response
             else:
-                self.view_finished.send(self, view=view, response=response)
+                self.view_finished.send(self, view=view, request=request,
+                                        response=response)
                 return response
 
         # A route was matched, but an error was returned
@@ -178,8 +176,10 @@ class FrescoApp(RouteCollection):
         """\
         Add a WSGI middleware layer
         """
+        if self._wsgi_app:
+            raise AssertionError("Cannot add middleware: "
+                                 "application is now receiving requests")
         self._middleware.append((middleware, args, kwargs))
-        self._wsgi_app = self._make_wsgi_app()
 
     def _raw_wsgi_app(self, environ, start_response):
         return self.view(self, self.request_class(environ))(environ,
@@ -281,7 +281,9 @@ class FrescoApp(RouteCollection):
         for key, value in kwargs.items():
             key = key.replace('wsgi_', 'wsgi.')
             if '.' not in key:
-                value = unicode_to_environ(value)
+                # value must be a python native str, whatever that means
+                if not isinstance(value, str):
+                    value = unicode_to_environ(value)
             env_overrides[key] = value
 
         environ = {
@@ -312,24 +314,23 @@ class FrescoApp(RouteCollection):
             environ['QUERY_STRING'] = url.query
 
         environ.update(env_overrides)
-        context.push(request=self.request_class(environ), app=self)
 
-        def _fakewsgiapp(environ, start_response):
-            start_response('200 OK', [])
-            yield b''
-        _fakewsgiapp = self._make_wsgi_app(_fakewsgiapp)
+        if getattr(self, '_fakewsgiapp', None) is None:
+            def _fakewsgiapp(environ, start_response):
+                start_response('200 OK', [])
+                yield b''
+            self._fakewsgiapp = self._make_wsgi_app(_fakewsgiapp)
 
         # Invoke a fake wsgi app wrapped in all middleware layers. This
         # executes any middleware, which may have side effects which are
         # expected downstream (eg setting environ keys)
-        wsgi_iter = _fakewsgiapp(environ, lambda *a, **kw: None)
+        wsgi_iter = self._fakewsgiapp(environ, lambda *a, **kw: None)
         next(wsgi_iter)
         yield context
 
         # Exhaust the iterator and close up
         list(wsgi_iter)
         getattr(wsgi_iter, 'close', lambda: None)()
-        context.pop()
 
 
 def collect_keys(items):
