@@ -6,6 +6,7 @@ The :class:`Request` class models an incoming HTTP request, allowing access to
 HTTP headers and request data (eg query string or submitted form data).
 """
 
+import json
 import posixpath
 import re
 
@@ -39,6 +40,8 @@ class Request(object):
     #: Maximum size for multipart/form-data encoded post data
     MAX_MULTIPART_SIZE = 2 * MB
 
+    _body_bytes = None
+    _parsed_content_type = None
     _form = None
     _files = None
     _query = None
@@ -54,6 +57,9 @@ class Request(object):
     #: Encoding used to decode WSGI parameters, notably PATH_INFO and form data
     charset = fresco.DEFAULT_CHARSET
 
+    #: The decoder class to use for JSON request payloads
+    json_decoder_class = json.JSONDecoder
+
     def __new__(
         cls,
         environ,
@@ -93,20 +99,61 @@ class Request(object):
         if self._form is None:
             if self.environ['REQUEST_METHOD'] in ('PUT', 'POST'):
                 self._form = MultiDict(
-                    parse_post(
-                        self.environ,
-                        self.environ['wsgi.input'],
-                        self.charset,
-                        self.MAX_SIZE,
-                        self.MAX_MULTIPART_SIZE,
-                    )
+                    parse_post(self.environ,
+                               self.environ['wsgi.input'],
+                               self.charset,
+                               self.MAX_SIZE,
+                               self.MAX_MULTIPART_SIZE)
                 )
             else:
-                query = environ_to_unicode(self.environ.get('QUERY_STRING',
-                                                            ''))
+                query = environ_to_unicode(
+                            self.environ.get('QUERY_STRING', ''))
                 self._form = MultiDict(parse_querystring(query, self.charset))
         return self._form
 
+    @property
+    def content_type_encoding(self):
+        if self._parsed_content_type is None:
+            self._parsed_content_type = get_content_type_info(self.environ)
+        return self._parsed_content_type.encoding
+
+    @property
+    def content_type(self):
+        if self._parsed_content_type is None:
+            self._parsed_content_type = get_content_type_info(self.environ)
+        return self._parsed_content_type.content_type
+
+    @property
+    def body_bytes(self):
+        if self._body_bytes is None:
+            self._body_bytes = get_body_bytes(self.environ, self.MAX_SIZE)
+        return self._body_bytes
+
+    @property
+    def body(self):
+        body = self.body_bytes
+        if body is None:
+            return None
+        encoding = self.content_type_encoding or self.charset
+        try:
+            return body.decode(encoding)
+        except UnicodeDecodeError:
+            raise exceptions.RequestParseError(
+                "Payload contains data that could not be decoded using " +
+                encoding)
+
+    def get_json(self, *args, **kwargs):
+        """
+        Return a decoded JSON request body.
+
+        ``*args`` and ``**kwargs`` are passed to the ``JSONDecoder``
+        constructor.
+
+        This will try to decode a json string regardless of the mime type sent
+        by the client.
+        """
+        return self.json_decoder_class(*args, **kwargs).decode(self.body)
+
     @property
     def files(self):
         """
@@ -640,4 +687,5 @@ def currentrequest():
     except AttributeError:
         return None
 
-from .util.http import FileUpload, parse_querystring, parse_post
+from .util.http import (FileUpload, parse_querystring, parse_post,
+                        get_content_type_info, get_body_bytes)
