@@ -10,23 +10,30 @@ import fresco
 from .cookie import Cookie
 
 __all__ = [
-    'STATUS_CONTINUE', 'STATUS_SWITCHING_PROTOCOLS',
+    'STATUS_CONTINUE', 'STATUS_SWITCHING_PROTOCOLS', 'STATUS_PROCESSING',
     'STATUS_OK', 'STATUS_CREATED', 'STATUS_ACCEPTED',
     'STATUS_NON_AUTHORITATIVE_INFORMATION', 'STATUS_NO_CONTENT',
-    'STATUS_RESET_CONTENT', 'STATUS_PARTIAL_CONTENT',
-    'STATUS_MULTIPLE_CHOICES', 'STATUS_MOVED_PERMANENTLY', 'STATUS_FOUND',
-    'STATUS_SEE_OTHER', 'STATUS_NOT_MODIFIED', 'STATUS_USE_PROXY',
-    'STATUS_TEMPORARY_REDIRECT', 'STATUS_BAD_REQUEST', 'STATUS_UNAUTHORIZED',
+    'STATUS_RESET_CONTENT', 'STATUS_PARTIAL_CONTENT', 'STATUS_MULTI_STATUS',
+    'STATUS_ALREADY_REPORTED', 'STATUS_IM_USED', 'STATUS_MULTIPLE_CHOICES',
+    'STATUS_MOVED_PERMANENTLY', 'STATUS_FOUND', 'STATUS_SEE_OTHER',
+    'STATUS_NOT_MODIFIED', 'STATUS_USE_PROXY', 'STATUS_TEMPORARY_REDIRECT',
+    'STATUS_PERMANENT_REDIRECT', 'STATUS_BAD_REQUEST', 'STATUS_UNAUTHORIZED',
     'STATUS_PAYMENT_REQUIRED', 'STATUS_FORBIDDEN', 'STATUS_NOT_FOUND',
     'STATUS_METHOD_NOT_ALLOWED', 'STATUS_NOT_ACCEPTABLE',
-    'STATUS_PROXY_AUTHENTICATION_REQUIRED', 'STATUS_REQUEST_TIME_OUT',
+    'STATUS_PROXY_AUTHENTICATION_REQUIRED', 'STATUS_REQUEST_TIMEOUT',
     'STATUS_CONFLICT', 'STATUS_GONE', 'STATUS_LENGTH_REQUIRED',
-    'STATUS_PRECONDITION_FAILED', 'STATUS_REQUEST_ENTITY_TOO_LARGE',
-    'STATUS_REQUEST_URI_TOO_LARGE', 'STATUS_UNSUPPORTED_MEDIA_TYPE',
-    'STATUS_REQUESTED_RANGE_NOT_SATISFIABLE', 'STATUS_EXPECTATION_FAILED',
+    'STATUS_PRECONDITION_FAILED', 'STATUS_PAYLOAD_TOO_LARGE',
+    'STATUS_URI_TOO_LONG', 'STATUS_UNSUPPORTED_MEDIA_TYPE',
+    'STATUS_RANGE_NOT_SATISFIABLE', 'STATUS_EXPECTATION_FAILED',
+    'STATUS_UNPROCESSABLE_ENTITY', 'STATUS_LOCKED', 'STATUS_FAILED_DEPENDENCY',
+    'STATUS_UPGRADE_REQUIRED', 'STATUS_PRECONDITION_REQUIRED',
+    'STATUS_TOO_MANY_REQUESTS', 'STATUS_REQUEST_HEADER_FIELDS_TOO_LARGE',
     'STATUS_INTERNAL_SERVER_ERROR', 'STATUS_NOT_IMPLEMENTED',
     'STATUS_BAD_GATEWAY', 'STATUS_SERVICE_UNAVAILABLE',
-    'STATUS_GATEWAY_TIME_OUT', 'STATUS_HTTP_VERSION_NOT_SUPPORTED',
+    'STATUS_GATEWAY_TIMEOUT', 'STATUS_HTTP_VERSION_NOT_SUPPORTED',
+    'STATUS_VARIANT_ALSO_NEGOTIATES', 'STATUS_INSUFFICIENT_STORAGE',
+    'STATUS_LOOP_DETECTED', 'STATUS_NOT_EXTENDED',
+    'STATUS_NETWORK_AUTHENTICATION_REQUIRED',
     'Response'
 ]
 
@@ -34,6 +41,7 @@ __all__ = [
 #: HTTP/1.1 status codes as listed in http://www.ietf.org/rfc/rfc2616.txt
 HTTP_STATUS_CODES = {100: 'Continue',
                      101: 'Switching Protocols',
+                     102: 'Processing',
                      200: 'OK',
                      201: 'Created',
                      202: 'Accepted',
@@ -41,6 +49,9 @@ HTTP_STATUS_CODES = {100: 'Continue',
                      204: 'No Content',
                      205: 'Reset Content',
                      206: 'Partial Content',
+                     207: 'Multi-Status',
+                     208: 'Already Reported',
+                     226: 'IM Used',
                      300: 'Multiple Choices',
                      301: 'Moved Permanently',
                      302: 'Found',
@@ -48,6 +59,7 @@ HTTP_STATUS_CODES = {100: 'Continue',
                      304: 'Not Modified',
                      305: 'Use Proxy',
                      307: 'Temporary Redirect',
+                     308: 'Permanent Redirect',
                      400: 'Bad Request',
                      401: 'Unauthorized',
                      402: 'Payment Required',
@@ -56,27 +68,39 @@ HTTP_STATUS_CODES = {100: 'Continue',
                      405: 'Method Not Allowed',
                      406: 'Not Acceptable',
                      407: 'Proxy Authentication Required',
-                     408: 'Request Time-out',
+                     408: 'Request Timeout',
                      409: 'Conflict',
                      410: 'Gone',
                      411: 'Length Required',
                      412: 'Precondition Failed',
-                     413: 'Request Entity Too Large',
-                     414: 'Request-URI Too Large',
+                     413: 'Payload Too Large',
+                     414: 'URI Too Long',
                      415: 'Unsupported Media Type',
-                     416: 'Requested range not satisfiable',
+                     416: 'Range Not Satisfiable',
                      417: 'Expectation Failed',
+                     422: 'Unprocessable Entity',
+                     423: 'Locked',
+                     424: 'Failed Dependency',
+                     426: 'Upgrade Required',
+                     428: 'Precondition Required',
+                     429: 'Too Many Requests',
+                     431: 'Request Header Fields Too Large',
                      500: 'Internal Server Error',
                      501: 'Not Implemented',
                      502: 'Bad Gateway',
                      503: 'Service Unavailable',
-                     504: 'Gateway Time-out',
-                     505: 'HTTP Version not supported',
-                     }
+                     504: 'Gateway Timeout',
+                     505: 'HTTP Version Not Supported',
+                     506: 'Variant Also Negotiates',
+                     507: 'Insufficient Storage',
+                     508: 'Loop Detected',
+                     510: 'Not Extended',
+                     511: 'Network Authentication Required'}
 
 # Symbolic names for the HTTP status codes
 STATUS_CONTINUE = 100
 STATUS_SWITCHING_PROTOCOLS = 101
+STATUS_PROCESSING = 102
 STATUS_OK = 200
 STATUS_CREATED = 201
 STATUS_ACCEPTED = 202
@@ -84,6 +108,9 @@ STATUS_NON_AUTHORITATIVE_INFORMATION = 203
 STATUS_NO_CONTENT = 204
 STATUS_RESET_CONTENT = 205
 STATUS_PARTIAL_CONTENT = 206
+STATUS_MULTI_STATUS = 207
+STATUS_ALREADY_REPORTED = 208
+STATUS_IM_USED = 226
 STATUS_MULTIPLE_CHOICES = 300
 STATUS_MOVED_PERMANENTLY = 301
 STATUS_FOUND = 302
@@ -91,6 +118,7 @@ STATUS_SEE_OTHER = 303
 STATUS_NOT_MODIFIED = 304
 STATUS_USE_PROXY = 305
 STATUS_TEMPORARY_REDIRECT = 307
+STATUS_PERMANENT_REDIRECT = 308
 STATUS_BAD_REQUEST = 400
 STATUS_UNAUTHORIZED = 401
 STATUS_PAYMENT_REQUIRED = 402
@@ -99,23 +127,34 @@ STATUS_NOT_FOUND = 404
 STATUS_METHOD_NOT_ALLOWED = 405
 STATUS_NOT_ACCEPTABLE = 406
 STATUS_PROXY_AUTHENTICATION_REQUIRED = 407
-STATUS_REQUEST_TIME_OUT = 408
+STATUS_REQUEST_TIMEOUT = 408
 STATUS_CONFLICT = 409
 STATUS_GONE = 410
 STATUS_LENGTH_REQUIRED = 411
 STATUS_PRECONDITION_FAILED = 412
-STATUS_REQUEST_ENTITY_TOO_LARGE = 413
-STATUS_REQUEST_URI_TOO_LARGE = 414
+STATUS_PAYLOAD_TOO_LARGE = 413
+STATUS_URI_TOO_LONG = 414
 STATUS_UNSUPPORTED_MEDIA_TYPE = 415
-STATUS_REQUESTED_RANGE_NOT_SATISFIABLE = 416
+STATUS_RANGE_NOT_SATISFIABLE = 416
 STATUS_EXPECTATION_FAILED = 417
+STATUS_UNPROCESSABLE_ENTITY = 422
+STATUS_LOCKED = 423
+STATUS_FAILED_DEPENDENCY = 424
+STATUS_UPGRADE_REQUIRED = 426
+STATUS_PRECONDITION_REQUIRED = 428
+STATUS_TOO_MANY_REQUESTS = 429
+STATUS_REQUEST_HEADER_FIELDS_TOO_LARGE = 431
 STATUS_INTERNAL_SERVER_ERROR = 500
 STATUS_NOT_IMPLEMENTED = 501
 STATUS_BAD_GATEWAY = 502
 STATUS_SERVICE_UNAVAILABLE = 503
-STATUS_GATEWAY_TIME_OUT = 504
+STATUS_GATEWAY_TIMEOUT = 504
 STATUS_HTTP_VERSION_NOT_SUPPORTED = 505
-
+STATUS_VARIANT_ALSO_NEGOTIATES = 506
+STATUS_INSUFFICIENT_STORAGE = 507
+STATUS_LOOP_DETECTED = 508
+STATUS_NOT_EXTENDED = 510
+STATUS_NETWORK_AUTHENTICATION_REQUIRED = 511
 
 #: Mapping from python symbolic names to HTTP headers to ensure headers are
 #: emitted with correct capitalization
@@ -636,21 +675,23 @@ class Response(object):
         )
 
     @classmethod
-    def request_entity_too_large(cls, request=None):
+    def payload_too_large(cls, request=None):
         """\
-        Return an HTTP Request Entity Too Large response (413)::
+        Return an HTTP Payload Too Large response (413)::
 
-            >>> response = Response.request_entity_too_large()
+            >>> response = Response.payload_too_large()
 
         """
         return cls(
-            status=STATUS_REQUEST_ENTITY_TOO_LARGE,
+            status=STATUS_PAYLOAD_TOO_LARGE,
             content=["<html>"
                      "<body>"
-                     "<h1>Request Entity Too Large</h1>"
+                     "<h1>Payload Too Large</h1>"
                      "</body>"
                      "</html>"])
 
+    request_entity_too_large = payload_too_large
+
     @classmethod
     def method_not_allowed(cls, valid_methods):
         """\
@@ -697,8 +738,7 @@ class Response(object):
                      "<body>"
                      "<h1>Internal Server Error</h1>"
                      "</body>"
-                     "</html>"
-                    ])
+                     "</html>"])
 
     @classmethod
     def redirect(cls, location, request=None, status=STATUS_FOUND):
