@@ -1,5 +1,5 @@
 from mock import Mock
-from nose.tools import assert_equal, assert_raises
+import pytest
 
 from fresco import Response
 from fresco import FrescoApp, GET, POST, PUT, DELETE
@@ -274,9 +274,9 @@ class TestViewCollection(object):
         app.include('/baz', baz)
 
         with app.requestcontext():
-            assert_equal(urlfor(foo.index_html), 'http://localhost/')
-            assert_equal(urlfor(bar.index_html), 'http://localhost/bar/')
-            assert_equal(urlfor(baz.index_html), 'http://localhost/baz/')
+            assert urlfor(foo.index_html) == 'http://localhost/'
+            assert urlfor(bar.index_html) == 'http://localhost/bar/'
+            assert urlfor(baz.index_html) == 'http://localhost/baz/'
 
     def test_instance_available_in_context(self):
 
@@ -398,7 +398,8 @@ class TestAppRequestContext(object):
         env = {}
         setup_testing_defaults(env)
         app(env, Mock()).close()
-        assert_raises(AssertionError, app.add_middleware, Mock())
+        with pytest.raises(AssertionError):
+            app.add_middleware(Mock())
 
 
 class TestResponseExceptions(object):
