@@ -1,4 +1,5 @@
 # encoding: utf-8
+from copy import copy
 from functools import wraps
 
 from mock import Mock
@@ -11,7 +12,8 @@ from fresco.exceptions import NotFound
 from fresco.compat import ustring_type
 from fresco.core import urlfor
 from fresco.routing import (
-    Route, DelegateRoute, RouteCollection, routefor, RouteTraversal)
+    Route, DelegateRoute, RouteCollection, routefor, RouteTraversal,
+    RouteNotFound)
 from . import fixtures
 
 
@@ -219,6 +221,76 @@ class TestRouteCollection(object):
         rc = RouteCollection([Route('/<:str>', GET, view)])
         assert rc.pathfor(view, 'x') == '/x'
 
+    def test_replace_raises_route_not_found(self):
+        a = RouteCollection()
+        view = Mock()
+        a.route('/harvey', GET, view, name='harvey')
+        with pytest.raises(RouteNotFound):
+            a.replace('rabbit', None)
+
+    def test_replace_selects_routes_by_name(self):
+        a = RouteCollection()
+        oldroute = Route('/', GET, Mock(), name='harvey')
+        newroute = Route('/', GET, Mock())
+        a.add_route(oldroute)
+        a.replace('harvey', newroute)
+        assert a.__routes__ == [newroute]
+
+    def test_replace_selects_routes_by_view(self):
+        a = RouteCollection()
+        view = Mock()
+        oldroute = Route('/', GET, view)
+        newroute = Route('/', GET, Mock())
+        a.add_route(oldroute)
+        a.replace(view, newroute)
+        assert a.__routes__ == [newroute]
+
+    def test_can_add_a_list_to_a_routecollection(self):
+        r1 = Route('/', GET, Mock())
+        r2 = Route('/', GET, Mock())
+        assert (RouteCollection([r1]) + [r2]).__routes__ == [r1, r2]
+
+    def test_can_add_routecollections(self):
+        r1 = Route('/', GET, Mock())
+        r2 = Route('/', GET, Mock())
+        assert (RouteCollection([r1]) + RouteCollection([r2])).__routes__ == \
+                [r1, r2]
+
+    def test_routecollections_can_be_used_in_classes(self):
+        class MyViews(object):
+            __routes__ = RouteCollection([
+                Route('/', GET, 'view')])
+
+            def view(self):
+                pass
+
+        v = MyViews()
+        app = FrescoApp()
+        app.include('/', v)
+        assert [r.route.getview()
+                for r in app.get_routes('/', GET)] == [v.view]
+
+    def test_routecollections_in_classes_can_be_manipulated(self):
+        class MyViews(object):
+            __routes__ = RouteCollection([
+                Route('/', GET, 'view')])
+
+            def view(self):
+                pass
+
+        class MyOtherViews(MyViews):
+            __routes__ = copy(MyViews.__routes__)
+            __routes__.replace('view', Route('/', GET, 'another_view'))
+
+            def another_view(self):
+                pass
+
+        v = MyOtherViews()
+        app = FrescoApp()
+        app.include('/', v)
+        assert [r.route.getview()
+                for r in app.get_routes('/', GET)] == [v.another_view]
+
 
 class TestRoutefor(object):
 
