@@ -89,14 +89,14 @@ class TestParseMultipart(object):
             files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)],
             check_status=False
         )
-        assert r.response.status == "413 Request Entity Too Large"
+        assert r.response.status == "413 Payload Too Large"
 
         r = TestAgent(app).post_multipart(
             files=[('f1', 'filename.txt', 'text/plain', 'x' * 400)],
             data={'f2': 'x' * 101},
             check_status=False
         )
-        assert r.response.status == "413 Request Entity Too Large"
+        assert r.response.status == "413 Payload Too Large"
 
     def test_fileupload_with_invalid_content_length(self):
 
@@ -131,7 +131,7 @@ class TestParseMultipart(object):
 
         r = TestAgent(app).post_multipart(data=[('f1', 'x' * 200)],
                                           check_status=False)
-        assert r.response.status == "413 Request Entity Too Large"
+        assert r.response.status == "413 Payload Too Large"
 
 
 class TestParseFormEncodedData(object):
@@ -156,7 +156,7 @@ class TestParseFormEncodedData(object):
         app.route('/', POST, view)
 
         r = TestAgent(app).post(data=[('f1', 'x' * 200)], check_status=False)
-        assert r.response.status == "413 Request Entity Too Large"
+        assert r.response.status == "413 Payload Too Large"
 
     def test_posted_data_contains_non_ascii_chars(self):
         """
@@ -201,3 +201,21 @@ class TestParseFormEncodedData(object):
             request = c.request
             request.charset = 'latin1'
             assert request.query['char'] == self.char
+
+
+class TestParseBody(object):
+
+    def test_formencoded_data_too_big(self):
+        """
+        POST data raises an exception if it exceeds request.MAX_SIZE bytes
+        """
+        def view():
+            request = context.request
+            request.MAX_SIZE = 100
+            request.body
+            return Response(['ok'])
+        app = FrescoApp()
+        app.route('/', POST, view)
+
+        r = TestAgent(app).post(data=b'x' * 200, check_status=False)
+        assert r.response.status == "413 Payload Too Large"
